@@ -59,7 +59,6 @@ pub(crate) trait Node {
     fn base(&self) -> &BaseNode;
     fn base_mut(&mut self) -> &mut BaseNode;
     fn is_full(&self) -> bool;
-    fn is_under_full(&self) -> bool;
     fn insert(&mut self, key: u8, node: NodePtr);
     fn change(&mut self, key: u8, val: NodePtr) -> NodePtr;
     fn get_child(&self, key: u8) -> Option<NodePtr>;
@@ -67,6 +66,9 @@ pub(crate) trait Node {
     fn remove(&mut self, k: u8);
     fn copy_to<N: Node>(&self, dst: &mut N);
     fn get_type() -> NodeType;
+
+    #[cfg(feature = "db_extension")]
+    fn get_random_child(&self, rng: &mut impl rand::Rng) -> Option<(u8, NodePtr)>;
 }
 
 pub(crate) enum NodeIter<'a> {
@@ -103,13 +105,13 @@ pub(crate) struct NodeMeta {
     prefix: Prefix,
 }
 
-impl Drop for BaseNode {
-    fn drop(&mut self) {
-        let layout = self.get_type().node_layout();
-        unsafe {
-            std::alloc::dealloc(self as *mut BaseNode as *mut u8, layout);
-        }
-    }
+#[cfg(test)]
+mod const_assert {
+    use super::*;
+    static_assertions::const_assert_eq!(std::mem::size_of::<NodeMeta>(), 16);
+    static_assertions::const_assert_eq!(std::mem::align_of::<NodeMeta>(), 4);
+    static_assertions::const_assert_eq!(std::mem::size_of::<BaseNode>(), 24);
+    static_assertions::const_assert_eq!(std::mem::align_of::<BaseNode>(), 8);
 }
 
 macro_rules! gen_method {
@@ -168,6 +170,13 @@ macro_rules! gen_method_mut {
 
 gen_method!(get_child, (k: u8), Option<NodePtr>);
 gen_method!(get_children, (start: u8, end: u8), NodeIter<'_>);
+
+#[cfg(feature = "db_extension")]
+gen_method!(
+    get_random_child,
+    (rng: &mut impl rand::Rng),
+    Option<(u8, NodePtr)>
+);
 gen_method_mut!(change, (key: u8, val: NodePtr), NodePtr);
 gen_method_mut!(remove, (key: u8), ());
 
@@ -194,7 +203,7 @@ impl BaseNode {
         }
     }
 
-    pub(crate) fn make_node<N: Node>(prefix: &[u8]) -> Box<N> {
+    pub(crate) fn make_node<N: Node>(prefix: &[u8]) -> *mut N {
         let node = BaseNode::new(N::get_type(), prefix);
         let layout = N::get_type().node_layout();
         unsafe {
@@ -206,10 +215,14 @@ impl BaseNode {
                 (&mut *mem).init_empty();
             }
 
-            Box::from_raw(ptr as *mut N)
+            ptr as *mut N
         }
     }
 
+    pub(crate) unsafe fn drop_node(node: *mut BaseNode) {
+        std::alloc::dealloc(node as *mut u8, (&*node).get_type().node_layout());
+    }
+
     pub(crate) fn get_type(&self) -> NodeType {
         self.meta.node_type
     }
@@ -225,6 +238,10 @@ impl BaseNode {
 
     pub(crate) fn read_lock(&self) -> Result<ReadGuard, ArtError> {
         let version = self.type_version_lock_obsolete.load(Ordering::Acquire);
+
+        #[cfg(test)]
+        crate::utils::fail_point(ArtError::Locked(version))?;
+
         if Self::is_locked(version) || Self::is_obsolete(version) {
             return Err(ArtError::Locked(version));
         }
@@ -278,20 +295,19 @@ impl BaseNode {
 
         let mut write_n = n.upgrade().map_err(|v| v.1)?;
 
-        let mut n_big = BaseNode::make_node::<BiggerT>(write_n.as_ref().base().prefix());
-        write_n.as_ref().copy_to(n_big.as_mut());
-        n_big.insert(key, val);
+        let n_big = BaseNode::make_node::<BiggerT>(write_n.as_ref().base().prefix());
+        write_n.as_ref().copy_to(unsafe { &mut *n_big });
+        unsafe { &mut *n_big }.insert(key, val);
 
-        write_p.as_mut().change(
-            key_parent,
-            NodePtr::from_node(Box::into_raw(n_big) as *mut BaseNode),
-        );
+        write_p
+            .as_mut()
+            .change(key_parent, NodePtr::from_node(n_big as *mut BaseNode));
 
         write_n.mark_obsolete();
         let delete_n = write_n.as_mut() as *mut CurT as usize;
         std::mem::forget(write_n);
         guard.defer(move || unsafe {
-            std::ptr::drop_in_place(delete_n as *mut BaseNode);
+            BaseNode::drop_node(delete_n as *mut BaseNode);
         });
         Ok(())
     }
