@@ -95,10 +95,6 @@ impl Node for Node256 {
         false
     }
 
-    fn is_under_full(&self) -> bool {
-        self.base.meta.count == 37
-    }
-
     fn insert(&mut self, key: u8, node: NodePtr) {
         self.children[key as usize] = node;
         self.set_mask(key as usize);
@@ -123,4 +119,20 @@ impl Node for Node256 {
             None
         }
     }
+
+    #[cfg(feature = "db_extension")]
+    fn get_random_child(&self, rng: &mut impl rand::Rng) -> Option<(u8, NodePtr)> {
+        if self.base.meta.count == 0 {
+            return None;
+        }
+        let mut scan_cnt = rng.gen_range(1..=self.base.meta.count);
+        let mut idx = 0;
+        while scan_cnt > 0 {
+            if self.get_mask(idx) {
+                scan_cnt -= 1;
+            }
+            idx += 1;
+        }
+        Some(((idx - 1) as u8, self.children[idx - 1]))
+    }
 }
