@@ -1,4 +1,4 @@
-use std::{marker::PhantomData, mem::ManuallyDrop};
+use std::marker::PhantomData;
 
 use crossbeam_epoch::Guard;
 
@@ -16,13 +16,14 @@ use crate::{
 /// Raw interface to the ART tree.
 /// The `Art` is a wrapper around the `RawArt` that provides a safe interface.
 /// Unlike `Art`, it support arbitrary `Key` types, see also `RawKey`.
-pub struct RawTree<K: RawKey> {
-    // use ManuallyDrop to avoid calling drop on the root node:
-    // On drop(), the Box will try deallocate the memory BaseNode
-    pub(crate) root: ManuallyDrop<Box<Node256>>,
+pub(crate) struct RawTree<K: RawKey> {
+    pub(crate) root: *const Node256,
     _pt_key: PhantomData<K>,
 }
 
+unsafe impl<K: RawKey> Send for RawTree<K> {}
+unsafe impl<K: RawKey> Sync for RawTree<K> {}
+
 impl<K: RawKey> Default for RawTree<K> {
     fn default() -> Self {
         Self::new()
@@ -31,8 +32,7 @@ impl<K: RawKey> Default for RawTree<K> {
 
 impl<T: RawKey> Drop for RawTree<T> {
     fn drop(&mut self) {
-        let v = unsafe { ManuallyDrop::take(&mut self.root) };
-        let mut sub_nodes = vec![(Box::into_raw(v) as *const BaseNode, 0)];
+        let mut sub_nodes = vec![(self.root as *const BaseNode, 0)];
 
         while !sub_nodes.is_empty() {
             let (node, level) = sub_nodes.pop().unwrap();
@@ -47,7 +47,7 @@ impl<T: RawKey> Drop for RawTree<T> {
                 }
             }
             unsafe {
-                std::ptr::drop_in_place(node as *mut BaseNode);
+                BaseNode::drop_node(node as *mut BaseNode);
             }
         }
     }
@@ -56,7 +56,7 @@ impl<T: RawKey> Drop for RawTree<T> {
 impl<T: RawKey> RawTree<T> {
     pub fn new() -> Self {
         RawTree {
-            root: ManuallyDrop::new(BaseNode::make_node::<Node256>(&[])),
+            root: BaseNode::make_node::<Node256>(&[]),
             _pt_key: PhantomData,
         }
     }
@@ -64,12 +64,12 @@ impl<T: RawKey> RawTree<T> {
 
 impl<T: RawKey> RawTree<T> {
     #[inline]
-    pub fn get(&self, key: &T, _guard: &Guard) -> Option<usize> {
+    pub(crate) fn get(&self, key: &T, _guard: &Guard) -> Option<usize> {
         'outer: loop {
             let mut parent_node;
             let mut level = 0;
 
-            let mut node = if let Ok(v) = self.root.base().read_lock() {
+            let mut node = if let Ok(v) = unsafe { &*self.root }.base().read_lock() {
                 v
             } else {
                 continue;
@@ -86,7 +86,7 @@ impl<T: RawKey> RawTree<T> {
                 let child_node = parent_node
                     .as_ref()
                     .get_child(key.as_bytes()[level as usize]);
-                if ReadGuard::check_version(&parent_node).is_err() {
+                if parent_node.check_version().is_err() {
                     continue 'outer;
                 }
 
@@ -112,7 +112,7 @@ impl<T: RawKey> RawTree<T> {
     #[inline]
     fn insert_inner(&self, k: &T, tid: usize, guard: &Guard) -> Result<Option<usize>, ArtError> {
         let mut parent_node = None;
-        let mut next_node = self.root.as_ref() as *const Node256 as *const BaseNode;
+        let mut next_node = self.root as *const BaseNode;
         let mut parent_key: u8;
         let mut node_key: u8 = 0;
         let mut level = 0;
@@ -143,11 +143,12 @@ impl<T: RawKey> RawTree<T> {
                                 NodePtr::from_tid(tid)
                             } else {
                                 let new_prefix = k.as_bytes();
-                                let mut n4 = BaseNode::make_node::<Node4>(
+                                let n4 = BaseNode::make_node::<Node4>(
                                     &new_prefix[level as usize + 1..k.len() - 1],
                                 );
-                                n4.insert(k.as_bytes()[k.len() - 1], NodePtr::from_tid(tid));
-                                NodePtr::from_node(Box::into_raw(n4) as *mut BaseNode)
+                                unsafe { &mut *n4 }
+                                    .insert(k.as_bytes()[k.len() - 1], NodePtr::from_tid(tid));
+                                NodePtr::from_node(n4 as *mut BaseNode)
                             }
                         };
 
@@ -161,8 +162,7 @@ impl<T: RawKey> RawTree<T> {
                         ) {
                             if level != 7 {
                                 unsafe {
-                                    // TODO: this is UB
-                                    std::ptr::drop_in_place(new_leaf.as_ptr() as *mut BaseNode);
+                                    BaseNode::drop_node(new_leaf.as_ptr() as *mut BaseNode);
                                 }
                             }
                             return Err(e);
@@ -194,7 +194,7 @@ impl<T: RawKey> RawTree<T> {
                     let mut write_n = node.upgrade().map_err(|(_n, v)| v)?;
 
                     // 1) Create new node which will be parent of node, Set common prefix, level to this node
-                    let mut new_node = BaseNode::make_node::<Node4>(
+                    let new_node = BaseNode::make_node::<Node4>(
                         write_n
                             .as_ref()
                             .prefix_range(0..((next_level - level) as usize)),
@@ -203,27 +203,29 @@ impl<T: RawKey> RawTree<T> {
                     // 2)  add node and (tid, *k) as children
                     if next_level == 7 {
                         // this is the last key, just insert to node
-                        new_node.insert(k.as_bytes()[next_level as usize], NodePtr::from_tid(tid));
+                        unsafe { &mut *new_node }
+                            .insert(k.as_bytes()[next_level as usize], NodePtr::from_tid(tid));
                     } else {
                         // otherwise create a new node
-                        let mut single_new_node = BaseNode::make_node::<Node4>(
+                        let single_new_node = BaseNode::make_node::<Node4>(
                             &k.as_bytes()[(next_level as usize + 1)..k.len() - 1],
                         );
 
-                        single_new_node.insert(k.as_bytes()[k.len() - 1], NodePtr::from_tid(tid));
-                        new_node.insert(
+                        unsafe { &mut *single_new_node }
+                            .insert(k.as_bytes()[k.len() - 1], NodePtr::from_tid(tid));
+                        unsafe { &mut *new_node }.insert(
                             k.as_bytes()[next_level as usize],
-                            NodePtr::from_node(Box::into_raw(single_new_node) as *const BaseNode),
+                            NodePtr::from_node(single_new_node as *const BaseNode),
                         );
                     }
 
-                    new_node.insert(no_match_key, NodePtr::from_node(write_n.as_mut()));
+                    unsafe { &mut *new_node }
+                        .insert(no_match_key, NodePtr::from_node(write_n.as_mut()));
 
                     // 3) upgradeToWriteLockOrRestart, update parentNode to point to the new node, unlock
-                    write_p.as_mut().change(
-                        parent_key,
-                        NodePtr::from_node(Box::into_raw(new_node) as *mut BaseNode),
-                    );
+                    write_p
+                        .as_mut()
+                        .change(parent_key, NodePtr::from_node(new_node as *mut BaseNode));
 
                     // 4) update prefix of node, unlock
                     let prefix_len = write_n.as_ref().prefix().len();
@@ -238,7 +240,7 @@ impl<T: RawKey> RawTree<T> {
     }
 
     #[inline]
-    pub fn insert(&self, k: T, tid: usize, guard: &Guard) -> Option<usize> {
+    pub(crate) fn insert(&self, k: T, tid: usize, guard: &Guard) -> Option<usize> {
         let backoff = Backoff::new();
         loop {
             match self.insert_inner(&k, tid, guard) {
@@ -296,19 +298,14 @@ impl<T: RawKey> RawTree<T> {
     }
 
     #[inline]
-    pub fn range(
+    pub(crate) fn range(
         &self,
         start: &T,
         end: &T,
         result: &mut [(usize, usize)],
         _guard: &Guard,
     ) -> usize {
-        let mut range_scan = RangeScan::new(
-            start,
-            end,
-            result,
-            self.root.as_ref() as *const Node256 as *const BaseNode,
-        );
+        let mut range_scan = RangeScan::new(start, end, result, self.root as *const BaseNode);
 
         if !range_scan.is_valid_key_pair() {
             return 0;
@@ -329,7 +326,7 @@ impl<T: RawKey> RawTree<T> {
     }
 
     pub(crate) fn remove_inner(&self, k: &T, guard: &Guard) -> Result<Option<usize>, ArtError> {
-        let mut next_node = self.root.as_ref() as *const Node256 as *const BaseNode;
+        let mut next_node = self.root as *const BaseNode;
         let mut parent_node: Option<ReadGuard> = None;
 
         let mut parent_key: u8;
@@ -385,7 +382,7 @@ impl<T: RawKey> RawTree<T> {
 
                             write_n.mark_obsolete();
                             guard.defer(move || unsafe {
-                                std::ptr::drop_in_place(write_n.as_mut());
+                                BaseNode::drop_node(write_n.as_mut());
                                 std::mem::forget(write_n);
                             });
                         } else {
@@ -407,7 +404,7 @@ impl<T: RawKey> RawTree<T> {
     }
 
     #[inline]
-    pub fn remove(&self, k: &T, guard: &Guard) -> Option<usize> {
+    pub(crate) fn remove(&self, k: &T, guard: &Guard) -> Option<usize> {
         let backoff = Backoff::new();
         loop {
             match self.remove_inner(k, guard) {
@@ -416,4 +413,149 @@ impl<T: RawKey> RawTree<T> {
             }
         }
     }
+
+    #[inline]
+    fn compute_if_present_inner<F>(
+        &self,
+        k: &T,
+        remapping_function: &mut F,
+        _guard: &Guard,
+    ) -> Result<Option<(usize, usize)>, ArtError>
+    where
+        F: FnMut(usize) -> usize,
+    {
+        let mut parent_node;
+        let mut level = 0;
+
+        let mut node = unsafe { &*self.root }.base().read_lock()?;
+
+        loop {
+            level = if let Some(v) = Self::check_prefix(node.as_ref(), k, level) {
+                v
+            } else {
+                return Ok(None);
+            };
+
+            if k.len() <= level as usize {
+                return Ok(None);
+            }
+
+            parent_node = node;
+            let child_node = parent_node.as_ref().get_child(k.as_bytes()[level as usize]);
+            parent_node.check_version()?;
+
+            let child_node = match child_node {
+                Some(n) => n,
+                None => return Ok(None),
+            };
+
+            if level == 7 {
+                let tid = child_node.as_tid();
+                let new_v = remapping_function(tid);
+                if new_v == tid {
+                    // the value is not change, early return;
+                    return Ok(Some((tid, tid)));
+                }
+                let mut write_n = parent_node.upgrade().map_err(|(_n, v)| v)?;
+                let old = write_n
+                    .as_mut()
+                    .change(k.as_bytes()[level as usize], NodePtr::from_tid(new_v));
+
+                debug_assert_eq!(tid, old.as_tid());
+
+                return Ok(Some((old.as_tid(), new_v)));
+            }
+
+            level += 1;
+
+            node = unsafe { &*child_node.as_ptr() }.read_lock()?;
+        }
+    }
+
+    #[inline]
+    pub(crate) fn compute_if_present<F>(
+        &self,
+        k: &T,
+        remapping_function: &mut F,
+        guard: &Guard,
+    ) -> Option<(usize, usize)>
+    where
+        F: FnMut(usize) -> usize,
+    {
+        let backoff = Backoff::new();
+        loop {
+            match self.compute_if_present_inner(k, &mut *remapping_function, guard) {
+                Ok(n) => return n,
+                Err(_) => backoff.spin(),
+            }
+        }
+    }
+
+    #[inline]
+    #[cfg(feature = "db_extension")]
+    pub(crate) fn compute_on_random(
+        &self,
+        rng: &mut impl rand::Rng,
+        f: &mut impl FnMut(usize, usize) -> usize,
+        guard: &Guard,
+    ) -> Option<(usize, usize, usize)> {
+        let backoff = Backoff::new();
+        loop {
+            match self.compute_on_random_inner(rng, f, guard) {
+                Ok(n) => return n,
+                Err(_) => backoff.spin(),
+            }
+        }
+    }
+
+    #[inline]
+    #[cfg(feature = "db_extension")]
+    fn compute_on_random_inner(
+        &self,
+        rng: &mut impl rand::Rng,
+        f: &mut impl FnMut(usize, usize) -> usize,
+        _guard: &Guard,
+    ) -> Result<Option<(usize, usize, usize)>, ArtError> {
+        let mut node = unsafe { &*self.root }.base().read_lock()?;
+
+        let mut key_tracker = crate::utils::KeyTracker::default();
+
+        loop {
+            for k in node.as_ref().prefix() {
+                key_tracker.push(*k);
+            }
+
+            let child_node = node.as_ref().get_random_child(rng);
+            node.check_version()?;
+
+            let (k, child_node) = match child_node {
+                Some(n) => n,
+                None => return Ok(None),
+            };
+
+            key_tracker.push(k);
+
+            if key_tracker.len() == 8 {
+                let new_v = f(key_tracker.to_usize_key(), child_node.as_tid());
+                if new_v == child_node.as_tid() {
+                    // Don't acquire the lock if the value is not changed
+                    return Ok(Some((key_tracker.to_usize_key(), new_v, new_v)));
+                }
+
+                let mut write_n = node.upgrade().map_err(|(_n, v)| v)?;
+
+                let old_v = write_n.as_mut().change(k, NodePtr::from_tid(new_v));
+
+                debug_assert_eq!(old_v.as_tid(), child_node.as_tid());
+
+                return Ok(Some((
+                    key_tracker.to_usize_key(),
+                    child_node.as_tid(),
+                    new_v,
+                )));
+            }
+
+            node = unsafe { &*child_node.as_ptr() }.read_lock()?;
+        }
+    }
 }
